# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-26 17:17
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0011_auto_20170326_1955'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='collection',
            name='exclude_properties_filters',
        ),
    ]
