# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-12 17:31
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('pcart_catalog', '0008_productimage_downloaded'),
    ]

    operations = [
        migrations.CreateModel(
            name='CollectionPluginModel',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='pcart_catalog_collectionpluginmodel', serialize=False, to='cms.CMSPlugin')),
                ('filter_string', models.CharField(blank=True, default='', max_length=255, verbose_name='Filter string')),
                ('sorting', models.CharField(blank=True, default='cheap', max_length=100, verbose_name='Sorting')),
                ('template_name', models.CharField(default='plugin', max_length=100, verbose_name='Template name')),
                ('collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='pcart_catalog.Collection', verbose_name='Collection')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]
