# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-20 18:32
from __future__ import unicode_literals

from django.db import migrations, models
import pcart_catalog.models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0003_auto_20170209_2135'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='producttype',
            options={'ordering': ['title'], 'verbose_name': 'Product type', 'verbose_name_plural': 'Product types'},
        ),
        migrations.AlterModelOptions(
            name='producttypeproperty',
            options={'ordering': ['position'], 'verbose_name': 'Product type property', 'verbose_name_plural': 'Product type properties'},
        ),
        migrations.AddField(
            model_name='collection',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to=pcart_catalog.models.generate_collection_image_filename, verbose_name='Image'),
        ),
        migrations.AddField(
            model_name='producttypeproperty',
            name='position',
            field=models.PositiveIntegerField(default=0, verbose_name='Position'),
        ),
        migrations.AddField(
            model_name='productvariant',
            name='compare_at_price',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=10, verbose_name='Compare at price'),
        ),
        migrations.AddField(
            model_name='vendor',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to=pcart_catalog.models.generate_vendor_image_filename, verbose_name='Image'),
        ),
    ]
