# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-09 09:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0005_auto_20170220_2047'),
    ]

    operations = [
        migrations.AddField(
            model_name='productimage',
            name='download_link',
            field=models.CharField(blank=True, default='', max_length=300, verbose_name='Download link'),
        ),
    ]
