# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-04-25 10:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0014_auto_20170411_1025'),
    ]

    operations = [
        migrations.AddField(
            model_name='vendor',
            name='slug',
            field=models.SlugField(max_length=255, null=True, verbose_name='Slug'),
        ),
    ]
