# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-09 09:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0006_productimage_download_link'),
    ]

    operations = [
        migrations.AddField(
            model_name='productimage',
            name='added',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now, verbose_name='Added'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='productimage',
            name='changed',
            field=models.DateTimeField(auto_now=True, verbose_name='Changed'),
        ),
    ]
