# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-05-05 07:04
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0017_auto_20170425_1325'),
    ]

    operations = [
        migrations.CreateModel(
            name='PropertyValueSlug',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('value', models.CharField(max_length=255, unique=True, verbose_name='Value')),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
            ],
            options={
                'ordering': ['value'],
                'verbose_name_plural': 'Property values slugs',
                'verbose_name': 'Property value slug',
            },
        ),
    ]
