# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-09 19:35
from __future__ import unicode_literals

from django.db import migrations

SQL = '''
CREATE INDEX idx_product_tags ON "pcart_catalog_product" USING GIN("tags");
CREATE INDEX idx_productvariant_tags ON "pcart_catalog_productvariant" USING GIN("tags");
'''

class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0002_auto_20170209_2130'),
    ]

    operations = [
        migrations.RunSQL(SQL),
    ]
