# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-26 16:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0010_auto_20170326_1953'),
    ]

    operations = [
        migrations.AlterField(
            model_name='collection',
            name='exclude_properties_filters',
            field=models.CharField(blank=True, default='', help_text='Comma separated list of properties tags which you want to exclude from filters.', max_length=255, verbose_name='Exclude properties filters'),
        ),
    ]
